<?xml version="1.0" encoding="iso-8859-1"?>

<!--
    Copyright (c) 2010, Alexis Royer, http://alexis.royer.free.fr/XmlModelBinding

    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

        * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
        * Neither the name of the XmlModelBinding project nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
    EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
    PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-->

<xsl:stylesheet
    version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xsd="http://www.w3.org/2001/XMLSchema"
    xmlns:xmb="http://alexis.royer.free.fr/XmlModelBinding">
<xsl:output method="text" encoding="iso-8859-1"/>

<!-- Parameters -->
<xsl:param name="xml2model" select="1"/>
<xsl:param name="dom" select="not($xml2model)"/>
<xsl:param name="debug" select="0"/>

<!-- Variables -->
<xsl:variable name="endl"><xsl:text>
</xsl:text></xsl:variable>
<xsl:variable name="model2xml" select="not($xml2model)"/>
<xsl:variable name="sax" select="not($dom)"/>
<xsl:variable name="indent"><xsl:text>    </xsl:text></xsl:variable>

<!-- Named templates -->
    <xsl:template name="autoIndent">
        <xsl:for-each select="parent::*"><xsl:call-template name="autoIndent"/></xsl:for-each>
        <!--<xsl:text>/*</xsl:text><xsl:value-of select="local-name()"/><xsl:text>*/</xsl:text>-->

        <xsl:choose>
        <xsl:when test="self::xsd:element[parent::xsd:schema]"><xsl:value-of select="$indent"/></xsl:when>
        <xsl:when test="self::xmb:call[ancestor::xsd:element[parent::xsd:schema]]"><xsl:value-of select="$indent"/></xsl:when>
        <xsl:when test="self::xsd:complexType[@name]"><xsl:value-of select="$indent"/></xsl:when>
        <xsl:when test="self::xsd:simpleType[@name]"><xsl:value-of select="$indent"/></xsl:when>
        <xsl:when test="self::xsd:attribute"><xsl:value-of select="$indent"/><xsl:if test="$xml2model = 1"><xsl:value-of select="$indent"/></xsl:if></xsl:when>
        <xsl:when test="self::xsd:element"><xsl:value-of select="$indent"/><xsl:if test="(@maxOccurs = 'unbounded') and ($model2xml = 1)"><xsl:value-of select="$indent"/></xsl:if></xsl:when>
        <xsl:when test="self::xsd:complexType"></xsl:when>
        <xsl:when test="self::xsd:sequence[parent::xsd:complexType]"><xsl:if test="$xml2model = 1"><xsl:value-of select="$indent"/></xsl:if></xsl:when>
        <xsl:when test="self::xsd:choice[parent::xsd:complexType]"><xsl:if test="$xml2model = 1"><xsl:value-of select="$indent"/></xsl:if></xsl:when>
        <xsl:when test="self::xsd:all[parent::xsd:complexType]"><xsl:if test="$xml2model = 1"><xsl:value-of select="$indent"/></xsl:if></xsl:when>
        <xsl:when test="self::xsd:enumeration"><xsl:value-of select="$indent"/><xsl:if test="$model2xml = 1"><xsl:value-of select="$indent"/></xsl:if></xsl:when>
        </xsl:choose>
    </xsl:template>
    <xsl:template name="indent0"><xsl:call-template name="autoIndent"/></xsl:template>
    <xsl:template name="indent1"><xsl:call-template name="indent0"/><xsl:value-of select="$indent"/></xsl:template>
    <xsl:template name="indent2"><xsl:call-template name="indent1"/><xsl:value-of select="$indent"/></xsl:template>
    <xsl:template name="indent3"><xsl:call-template name="indent2"/><xsl:value-of select="$indent"/></xsl:template>
    <xsl:template name="indent4"><xsl:call-template name="indent3"/><xsl:value-of select="$indent"/></xsl:template>
    <xsl:template name="indent5"><xsl:call-template name="indent4"/><xsl:value-of select="$indent"/></xsl:template>
    <xsl:template name="indent6"><xsl:call-template name="indent5"/><xsl:value-of select="$indent"/></xsl:template>
    <xsl:template name="indent7"><xsl:call-template name="indent6"/><xsl:value-of select="$indent"/></xsl:template>

    <xsl:template name="makeFunctionName">
        <xsl:choose>
        <xsl:when test="(self::xsd:complexType[@name] | self::xsd:simpleType[@name] | self::xsd:element[@name]) and parent::xsd:schema">
            <!-- Xml2Model or Model2Xml -->
            <xsl:choose>
            <xsl:when test="$xml2model = 1"><xsl:text>Xml2Model_</xsl:text></xsl:when>
            <xsl:when test="$model2xml = 1"><xsl:text>Model2Xml_</xsl:text></xsl:when>
            </xsl:choose>
            <!-- User type -->
            <xsl:choose>
            <xsl:when test="self::xsd:element">
                <xsl:choose>
                <xsl:when test="contains(@type,':')"><xsl:value-of select="substring-after(@type,':')"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="@type"/></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="self::xsd:complexType | self::xsd:simpleType">
                <xsl:value-of select="@name"/>
            </xsl:when>
            </xsl:choose>
        </xsl:when>
        <xsl:when test="self::xsd:element | self::xsd:attribute">
            <xsl:variable name="typeName">
                <xsl:choose>
                <xsl:when test="contains(@type,':')"><xsl:value-of select="substring-after(@type,':')"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="@type"/></xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:for-each select="/xsd:schema/xsd:complexType[@name=$typeName]"><xsl:call-template name="makeFunctionName"/></xsl:for-each>
            <xsl:for-each select="/xsd:schema/xsd:simpleType[@name=$typeName]"><xsl:call-template name="makeFunctionName"/></xsl:for-each>
        </xsl:when>
        <xsl:otherwise>
            <xsl:for-each select="parent::*">
                <xsl:call-template name="makeFunctionName"/>
            </xsl:for-each>
        </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="makeVarId">
        <xsl:choose>
        <xsl:when test="self::xsd:complexType"><xsl:value-of select="generate-id(.)"/></xsl:when>
        <xsl:when test="self::xsd:simpleType[@name]"><xsl:value-of select="generate-id(.)"/></xsl:when>
        <xsl:when test="self::xsd:element[parent::xsd:schema]"><xsl:value-of select="generate-id(.)"/></xsl:when>
        <xsl:otherwise><xsl:for-each select="parent::*"><xsl:call-template name="makeVarId"/></xsl:for-each></xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="traceTemplate">
        <xsl:if test="$debug = 1">
            <xsl:value-of select="$endl"/>
            <xsl:text>/* </xsl:text><xsl:value-of select="local-name()"/><xsl:text> */</xsl:text>
        </xsl:if>
    </xsl:template>

    <xsl:template name="startFunction">
        <xsl:call-template name="indent0"/> <xsl:text>void </xsl:text>
                                            <xsl:call-template name="makeFunctionName"/>
                                            <xsl:text>(</xsl:text>
                                                <xsl:if test="$dom = 1"><xsl:text>org.w3c.dom.Node xml_FocusNode</xsl:text><xsl:call-template name="makeVarId"/></xsl:if>
                                                <xsl:if test="$sax = 1"><xsl:text>SaxReader xmb_Sax</xsl:text></xsl:if>
                                                <xsl:for-each select="xmb:param">
                                                    <xsl:text>, </xsl:text><xsl:value-of select="."/>
                                                </xsl:for-each>
                                            <xsl:text>) {</xsl:text><xsl:value-of select="$endl"/>
        <xsl:call-template name="indent1"/>     <xsl:text>Trace.println("</xsl:text><xsl:call-template name="makeFunctionName"/><xsl:text>()");</xsl:text><xsl:value-of select="$endl"/>
    </xsl:template>

    <xsl:template name="endFunction">
        <xsl:call-template name="indent1"/>     <xsl:text>Trace.println("</xsl:text><xsl:call-template name="makeFunctionName"/><xsl:text>() ->");</xsl:text><xsl:value-of select="$endl"/>
        <xsl:call-template name="indent0"/> <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
        <xsl:value-of select="$endl"/>
    </xsl:template>

    <xsl:template name="complexTypeBody">
        <xsl:call-template name="complexTypeBody_init"/>

        <xsl:call-template name="complexTypeBody_globalCode_beforeAttributes"/>
        <xsl:call-template name="complexTypeBody_attributes"/>
        <xsl:call-template name="complexTypeBody_globalCode_afterAttributes"/>

        <xsl:call-template name="complexTypeBody_globalCode_beforeChilds"/>
        <xsl:call-template name="complexTypeBody_childs"/>
        <xsl:call-template name="complexTypeBody_globalCode_afterChilds"/>
    </xsl:template>
        <!-- Complex body initializations -->
        <xsl:template name="complexTypeBody_init">
            <xsl:if test="$dom = 1">
                <xsl:call-template name="indent1"/> <xsl:text>org.w3c.dom.Node xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/>
                                                        <xsl:text> = xml_FocusNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>;</xsl:text><xsl:value-of select="$endl"/>
            </xsl:if>
        </xsl:template>
        <!-- Global code -->
        <xsl:template name="complexTypeBody_globalCode_beforeAttributes">
            <xsl:for-each select="xmb:code[@location='before-attributes'] | xmb:xml2model[@location='before-attributes'] | xmb:model2xml[@location='before-attributes']">
                <xsl:apply-templates select="."/>
            </xsl:for-each>
            <xsl:value-of select="$endl"/>
        </xsl:template>
        <xsl:template name="complexTypeBody_globalCode_afterAttributes">
            <xsl:for-each select="xmb:code[@location='after-attributes'] | xmb:xml2model[@location='after-attributes'] | xmb:model2xml[@location='after-attributes']">
                <xsl:apply-templates select="."/>
            </xsl:for-each>
            <xsl:value-of select="$endl"/>
        </xsl:template>
        <xsl:template name="complexTypeBody_globalCode_beforeChilds">
            <xsl:for-each select="xmb:code[@location='before-childs'] | xmb:xml2model[@location='before-childs'] | xmb:model2xml[@location='before-childs']">
                <xsl:apply-templates select="."/>
            </xsl:for-each>
            <xsl:value-of select="$endl"/>
        </xsl:template>
        <xsl:template name="complexTypeBody_globalCode_afterChilds">
            <xsl:for-each select="xmb:code[@location='after-childs'] | xmb:xml2model[@location='after-childs'] | xmb:model2xml[@location='after-childs']">
                <xsl:apply-templates select="."/>
            </xsl:for-each>
            <xsl:value-of select="$endl"/>
        </xsl:template>
        <!-- Attributes -->
        <xsl:template name="complexTypeBody_attributes">
            <xsl:if test="$xml2model = 1"><xsl:call-template name="complexTypeBody_Xml2Model_attributes"/></xsl:if>
            <xsl:if test="$model2xml = 1"><xsl:call-template name="complexTypeBody_Model2Xml_attributes"/></xsl:if>
        </xsl:template>
        <!-- Read XML attributes -->
        <xsl:template name="complexTypeBody_Xml2Model_DOM_foreachAttribute">
            <xsl:call-template name="indent1"/> <xsl:text>for (</xsl:text>
                                                    <xsl:text>int i_Attr</xsl:text><xsl:call-template name="makeVarId"/><xsl:text> = 0; </xsl:text>
                                                    <xsl:text>i_Attr</xsl:text><xsl:call-template name="makeVarId"/>
                                                        <xsl:if test="$dom = 1"><xsl:text> &lt; xml_FocusNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>.getAttributes().getLength(); </xsl:text></xsl:if>
                                                        <xsl:if test="$sax = 1"><xsl:text> &lt; xmb_Sax.getAttributes().getLength(); </xsl:text></xsl:if>
                                                    <xsl:text>i_Attr</xsl:text><xsl:call-template name="makeVarId"/><xsl:text> ++</xsl:text>
                                                <xsl:text>) {</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/>
                                                        <xsl:text> = xml_FocusNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>.getAttributes().item(</xsl:text>
                                                            <xsl:text>i_Attr</xsl:text><xsl:call-template name="makeVarId"/>
                                                        <xsl:text>);</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>Trace.println(</xsl:text>
                                                        <xsl:text>"found attribute '" + xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/>
                                                        <xsl:text>.getNodeName() + "'"</xsl:text>
                                                    <xsl:text>);</xsl:text><xsl:value-of select="$endl"/>
        </xsl:template>
        <xsl:template name="complexTypeBody_Xml2Model_SAX_foreachAttribute">
            <xsl:call-template name="indent1"/> <xsl:text>for (</xsl:text>
                                                    <xsl:text>String j_AttrQName</xsl:text><xsl:call-template name="makeVarId"/><xsl:text> = xmb_Sax.getNextAttributeQName(); </xsl:text>
                                                    <xsl:text>j_AttrQName</xsl:text><xsl:call-template name="makeVarId"/><xsl:text> != null; </xsl:text>
                                                    <xsl:text>j_AttrQName</xsl:text><xsl:call-template name="makeVarId"/><xsl:text> = xmb_Sax.getNextAttributeQName()</xsl:text>
                                                <xsl:text>) {</xsl:text><xsl:value-of select="$endl"/>
        </xsl:template>
        <xsl:template name="complexTypeBody_Xml2Model_attributes">
            <xsl:if test="$dom = 1"><xsl:call-template name="complexTypeBody_Xml2Model_DOM_foreachAttribute"/></xsl:if>
            <xsl:if test="$sax = 1"><xsl:call-template name="complexTypeBody_Xml2Model_SAX_foreachAttribute"/></xsl:if>

                <xsl:call-template name="indent2"/> <xsl:text>if (false) {</xsl:text><xsl:value-of select="$endl"/>

                <!-- Scan all attributes and code directly attached to -->
                <xsl:for-each select="xsd:attribute | xmb:code[not(@location)] | xmb:xml2model[not(@location)]">
                    <xsl:apply-templates select="."/>
                </xsl:for-each>

                <xsl:call-template name="indent2"/> <xsl:text>}</xsl:text>
                                                    <xsl:value-of select="$endl"/>

                <xsl:call-template name="complexTypeBody_Xml2Model_attrResetCurrentNode"/>

            <xsl:call-template name="indent1"/> <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
            <xsl:value-of select="$endl"/>
        </xsl:template>
        <xsl:template name="complexTypeBody_Xml2Model_attrResetCurrentNode">
            <xsl:if test="$dom = 1">
                <xsl:call-template name="indent2"/> <xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/>
                                                        <xsl:text> = xml_FocusNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>;</xsl:text><xsl:value-of select="$endl"/>
            </xsl:if>
        </xsl:template>
        <!-- Write XML attributes -->
        <xsl:template name="complexTypeBody_Model2Xml_attributes">
            <xsl:variable name="foo" select="local-name()"/><!-- xsltproc curious patch -->

            <xsl:for-each select="xsd:attribute | xmb:code[not(@location)] | xmb:model2xml[not(@location)]">
                <xsl:apply-templates select="."/>
            </xsl:for-each>
            <xsl:if test="$dom = 1">
                <xsl:call-template name="indent1"/> <xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/>
                                                        <xsl:text> = xml_FocusNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>;</xsl:text><xsl:value-of select="$endl"/>
            </xsl:if>
        </xsl:template>
        <!-- Child nodes -->
        <xsl:template name="complexTypeBody_childs">
            <xsl:if test="$xml2model = 1"><xsl:call-template name="complexTypeBody_Xml2Model_childs"/></xsl:if>
            <xsl:if test="$model2xml = 1"><xsl:call-template name="complexTypeBody_Model2Xml_childs"/></xsl:if>
        </xsl:template>
        <!-- Read XML child nodes -->
        <xsl:template name="complexTypeBody_Xml2Model_childs">
            <xsl:if test="$dom = 1"><xsl:call-template name="complexTypeBody_Xml2Model_DOM_foreachChild"/></xsl:if>
            <xsl:if test="$sax = 1"><xsl:call-template name="complexTypeBody_Xml2Model_SAX_foreachChild"/></xsl:if>

                <xsl:call-template name="complexTypeBody_Xml2Model_childPreCode"/>

                <xsl:call-template name="indent2"/> <xsl:text>if (false) {</xsl:text><xsl:value-of select="$endl"/>
                    <xsl:call-template name="complexTypeBody_Xml2Model_childList"/>
                <xsl:call-template name="indent2"/> <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>

                <xsl:call-template name="complexTypeBody_Xml2Model_childPostCode"/>

                <xsl:call-template name="complexTypeBody_Xml2Model_childResetCurrentNode"/>

            <xsl:call-template name="indent1"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
        </xsl:template>
        <xsl:template name="complexTypeBody_Xml2Model_DOM_foreachChild">
            <xsl:call-template name="indent1"/> <xsl:text>for (</xsl:text>
                                                    <xsl:text>int i_Child</xsl:text><xsl:call-template name="makeVarId"/><xsl:text> = 0; </xsl:text>
                                                    <xsl:text>i_Child</xsl:text><xsl:call-template name="makeVarId"/>
                                                        <xsl:text> &lt; xml_FocusNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>.getChildNodes().getLength(); </xsl:text>
                                                    <xsl:text>i_Child</xsl:text><xsl:call-template name="makeVarId"/><xsl:text> ++</xsl:text>
                                                <xsl:text>) {</xsl:text><xsl:value-of select="$endl"/>

            <xsl:call-template name="indent2"/>     <xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/>
                                                        <xsl:text> = xml_FocusNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>.getChildNodes().item(</xsl:text>
                                                            <xsl:text>i_Child</xsl:text><xsl:call-template name="makeVarId"/>
                                                        <xsl:text>);</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>if (xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>.getNodeType()</xsl:text>
                                                        <xsl:text> == org.w3c.dom.Node.TEXT_NODE) </xsl:text>
                                                        <xsl:text>continue;</xsl:text><xsl:value-of select="$endl"/>
        </xsl:template>
        <xsl:template name="complexTypeBody_Xml2Model_SAX_foreachChild">
            <xsl:call-template name="indent1"/> <xsl:text>for (</xsl:text>
                                                    <xsl:text>String j_ChildQName</xsl:text><xsl:call-template name="makeVarId"/><xsl:text> = xmb_Sax.getNextChildQName(); </xsl:text>
                                                    <xsl:text>j_ChildQName</xsl:text><xsl:call-template name="makeVarId"/><xsl:text> != null; </xsl:text>
                                                    <xsl:text>j_ChildQName</xsl:text><xsl:call-template name="makeVarId"/><xsl:text> = xmb_Sax.getNextChildQName()</xsl:text>
                                                <xsl:text>) {</xsl:text><xsl:value-of select="$endl"/>
        </xsl:template>
        <xsl:template name="complexTypeBody_Xml2Model_childPreCode">
            <xsl:for-each select="*">
                <xsl:choose>
                <xsl:when test="self::xsd:sequence"><xsl:call-template name="complexTypeBody_Xml2Model_childPreCode"/></xsl:when>
                <xsl:when test="self::xsd:choice"><xsl:call-template name="complexTypeBody_Xml2Model_childPreCode"/></xsl:when>
                <xsl:when test="self::xsd:all"><xsl:call-template name="complexTypeBody_Xml2Model_childPreCode"/></xsl:when>
                <xsl:when test="self::xmb:code[not(parent::xsd:complexType) and (not(preceding-sibling::xsd:element) or following-sibling::xsd:element)]"><xsl:apply-templates select="."/></xsl:when>
                <xsl:when test="self::xmb:xml2model[not(parent::xsd:complexType) and (not(preceding-sibling::xsd:element) or following-sibling::xsd:element)]"><xsl:apply-templates select="."/></xsl:when>
                <xsl:otherwise></xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>
        </xsl:template>
        <xsl:template name="complexTypeBody_Xml2Model_childList">
            <xsl:for-each select="*">
                <xsl:choose>
                <xsl:when test="self::xsd:sequence"><xsl:call-template name="complexTypeBody_Xml2Model_childList"/></xsl:when>
                <xsl:when test="self::xsd:choice"><xsl:call-template name="complexTypeBody_Xml2Model_childList"/></xsl:when>
                <xsl:when test="self::xsd:all"><xsl:call-template name="complexTypeBody_Xml2Model_childList"/></xsl:when>
                <xsl:when test="self::xsd:element"><xsl:apply-templates select="."/></xsl:when>
                <xsl:otherwise></xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>
        </xsl:template>
        <xsl:template name="complexTypeBody_Xml2Model_childPostCode">
            <xsl:for-each select="*">
                <xsl:choose>
                <xsl:when test="self::xsd:sequence"><xsl:call-template name="complexTypeBody_Xml2Model_childPostCode"/></xsl:when>
                <xsl:when test="self::xsd:choice"><xsl:call-template name="complexTypeBody_Xml2Model_childPostCode"/></xsl:when>
                <xsl:when test="self::xsd:all"><xsl:call-template name="complexTypeBody_Xml2Model_childPostCode"/></xsl:when>
                <xsl:when test="self::xmb:code[not(parent::xsd:complexType) and not(following-sibling::xsd:element)]"><xsl:apply-templates select="."/></xsl:when>
                <xsl:when test="self::xmb:xml2model[not(parent::xsd:complexType) and not(following-sibling::xsd:element)]"><xsl:apply-templates select="."/></xsl:when>
                <xsl:otherwise></xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>
        </xsl:template>
        <xsl:template name="complexTypeBody_Xml2Model_childResetCurrentNode">
            <xsl:if test="$dom = 1">
                <xsl:call-template name="indent2"/> <xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/>
                                                        <xsl:text> = xml_FocusNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>;</xsl:text><xsl:value-of select="$endl"/>
            </xsl:if>
        </xsl:template>
        <!-- Write XML child nodes -->
        <xsl:template name="complexTypeBody_Model2Xml_childs">
            <xsl:for-each select="xsd:sequence | xsd:all | xsd:choice">
                <xsl:apply-templates select="."/>
                <xsl:if test="$dom = 1">
                    <xsl:call-template name="indent1"/> <xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/>
                                                            <xsl:text> = xml_FocusNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>;</xsl:text><xsl:value-of select="$endl"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:template>

<!-- XSD templates -->
    <xsl:template match="/xsd:schema">
        <xsl:call-template name="traceTemplate"/>
        <!-- User code -->
        <xsl:for-each select="*"><xsl:apply-templates select="."/></xsl:for-each>
        <!-- XML-tool functions -->
        <xsl:call-template name="xmlTools"/>
    </xsl:template>

    <xsl:template match="/xsd:schema/xsd:element">
        <xsl:call-template name="traceTemplate"/>

        <!-- Main function -->
        <xsl:call-template name="indent0"/> <xsl:text>void </xsl:text>
                                            <xsl:choose>
                                            <xsl:when test="$xml2model = 1"><xsl:text>xml2model</xsl:text></xsl:when>
                                            <xsl:when test="$model2xml = 1"><xsl:text>model2xml</xsl:text></xsl:when>
                                            </xsl:choose>
                                            <xsl:text>(</xsl:text>
                                                <xsl:text>java.io.File J_XmlFile</xsl:text>
                                                <xsl:for-each select="xmb:param">
                                                    <xsl:text>, </xsl:text><xsl:value-of select="."/>
                                                </xsl:for-each>
                                            <xsl:text>) </xsl:text>
                                            <xsl:text>throws org.xml.sax.SAXParseException </xsl:text>
                                            <xsl:text>{</xsl:text><xsl:value-of select="$endl"/>

        <xsl:call-template name="indent1"/>     <xsl:text>try {</xsl:text><xsl:value-of select="$endl"/>

        <xsl:if test="$xml2model = 1">
            <xsl:if test="$dom = 1">
                <xsl:call-template name="indent2"/>     <xsl:text>javax.xml.parsers.DocumentBuilderFactory xml_Factory = javax.xml.parsers.DocumentBuilderFactory.newInstance();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>javax.xml.parsers.DocumentBuilder xml_Parser = xml_Factory.newDocumentBuilder();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>org.w3c.dom.Document xml_Doc = xml_Parser.parse(new java.io.FileInputStream(J_XmlFile));</xsl:text><xsl:value-of select="$endl"/>

                <xsl:call-template name="indent2"/>     <xsl:text>org.w3c.dom.Node xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/>
                                                            <xsl:text> = xml_Doc.getFirstChild();</xsl:text><xsl:value-of select="$endl"/>
            </xsl:if>
            <xsl:if test="$sax = 1">
                <xsl:call-template name="indent2"/>     <xsl:text>SaxReader xmb_Sax = new SaxReader(this, new java.io.FileInputStream(J_XmlFile));</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>xmb_Sax.startParsing();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>xmb_Sax.getNextChildQName();</xsl:text><xsl:value-of select="$endl"/>
            </xsl:if>

                                                    <xsl:for-each select="*"><xsl:apply-templates select="."/></xsl:for-each>
            <xsl:if test="$dom = 1">
            </xsl:if>
            <xsl:if test="$sax = 1">
                <xsl:call-template name="indent2"/>     <xsl:text>xmb_Sax.stopParsing();</xsl:text><xsl:value-of select="$endl"/>
            </xsl:if>
        </xsl:if>

        <xsl:if test="$model2xml = 1">

            <xsl:call-template name="indent2"/>     <!-- Document Builder -->
                                                    <xsl:text>javax.xml.parsers.DocumentBuilderFactory xml_BuilderFactory = javax.xml.parsers.DocumentBuilderFactory.newInstance();</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>javax.xml.parsers.DocumentBuilder xml_Builder = xml_BuilderFactory.newDocumentBuilder();</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <!-- Document -->
                                                    <xsl:text>org.w3c.dom.Document xml_Doc = xml_Builder.newDocument();</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <!-- Root node creation -->
                                                    <xsl:text>org.w3c.dom.Element xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/>
                                                        <xsl:text> = xml_Doc.createElement(</xsl:text>
                                                            <xsl:text>"</xsl:text><xsl:value-of select="@name"/><xsl:text>"</xsl:text>
                                                        <xsl:text>);</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>xml_Doc.appendChild(xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>);</xsl:text><xsl:value-of select="$endl"/>

                                                    <!-- Feed the XML -->
                                                    <xsl:for-each select="*"><xsl:apply-templates select="."/></xsl:for-each>

            <xsl:call-template name="indent2"/>     <xsl:text>javax.xml.transform.TransformerFactory xsl_Factory = javax.xml.transform.TransformerFactory.newInstance();</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <!-- Identity transformation stylesheet. -->
                                                    <xsl:text>javax.xml.transform.Transformer xsl_Transformer = xsl_Factory.newTransformer();</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>xsl_Transformer.setOutputProperty(javax.xml.transform.OutputKeys.INDENT, "yes");</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>xsl_Transformer.setOutputProperty(javax.xml.transform.OutputKeys.ENCODING, "ISO-8859-1");</xsl:text><xsl:value-of select="$endl"/>

            <xsl:call-template name="indent2"/>     <!-- XML source -->
                                                    <xsl:text>javax.xml.transform.dom.DOMSource xml_Source = new javax.xml.transform.dom.DOMSource(xml_Doc);</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>javax.xml.transform.stream.StreamResult xml_Result = new javax.xml.transform.stream.StreamResult(</xsl:text>
                                                        <xsl:text>new java.io.FileOutputStream(J_XmlFile)</xsl:text>
                                                    <xsl:text>);</xsl:text><xsl:value-of select="$endl"/>

            <xsl:call-template name="indent2"/>     <!-- XSL Transformation -->
                                                    <xsl:text>xsl_Transformer.transform(xml_Source, xml_Result);</xsl:text><xsl:value-of select="$endl"/>
        </xsl:if>

        <xsl:call-template name="indent1"/>     <xsl:text>} catch (Exception j_Exception) {</xsl:text><xsl:value-of select="$endl"/>
        <xsl:call-template name="indent2"/>         <xsl:text>Trace.printStackTrace(j_Exception);</xsl:text><xsl:value-of select="$endl"/>
        <xsl:call-template name="indent1"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>

        <xsl:call-template name="indent0"/> <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
        <xsl:value-of select="$endl"/>
    </xsl:template>

    <xsl:template match="xsd:complexType[@name]">
        <xsl:call-template name="traceTemplate"/>

        <xsl:call-template name="startFunction"/>
        <xsl:call-template name="complexTypeBody"/>
        <xsl:call-template name="endFunction"/>
    </xsl:template>

    <xsl:template match="xsd:simpleType[@name]">
        <xsl:call-template name="traceTemplate"/>

        <xsl:call-template name="startFunction"/>
        <xsl:if test="$dom = 1">
            <xsl:call-template name="indent1"/> <xsl:text>org.w3c.dom.Node xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/>
                                                    <xsl:text> = xml_FocusNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>;</xsl:text><xsl:value-of select="$endl"/>
        </xsl:if>
        <xsl:for-each select="*"><xsl:apply-templates select="."/></xsl:for-each>
        <xsl:call-template name="endFunction"/>
    </xsl:template>

    <xsl:template match="xsd:complexType[not(@name)]">
        <xsl:call-template name="traceTemplate"/>

        <xsl:if test="$dom = 1">
            <xsl:call-template name="indent1"/> <xsl:text>org.w3c.dom.Node xml_FocusNode</xsl:text><xsl:call-template name="makeVarId"/>
                                                    <xsl:text> = </xsl:text><xsl:for-each select="parent::*"><xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/></xsl:for-each><xsl:text>;</xsl:text><xsl:value-of select="$endl"/>
        </xsl:if>
        <xsl:call-template name="complexTypeBody"/>
    </xsl:template>

    <xsl:template match="xsd:simpleType[not(@name)]">
        <xsl:call-template name="traceTemplate"/>

        <xsl:if test="($sax = 1) and not(ancestor::xsd:attribute)">
            <xsl:call-template name="complexTypeBody"/>
        </xsl:if>
        <xsl:for-each select="*"><xsl:apply-templates select="."/></xsl:for-each>
    </xsl:template>

    <xsl:template match="xsd:element">
        <xsl:call-template name="traceTemplate"/>

        <xsl:if test="$xml2model = 1">
            <xsl:call-template name="indent0"/> <xsl:text>} else if (</xsl:text>
                                                    <xsl:if test="$dom = 1"><xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>.getNodeName().equals(</xsl:text></xsl:if>
                                                    <xsl:if test="$sax = 1"><xsl:text>j_ChildQName</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>.equals(</xsl:text></xsl:if>
                                                        <xsl:text>"</xsl:text><xsl:value-of select="@name"/><xsl:text>"</xsl:text>
                                                    <xsl:text>)</xsl:text>
                                                <xsl:text>) {</xsl:text><xsl:value-of select="$endl"/>

                                                <xsl:for-each select="*"><xsl:apply-templates select="."/></xsl:for-each>
            <xsl:if test="($sax = 1) and not(.//xsd:complexType) and not(.//xmb:call)">
                <xsl:call-template name="indent1"/> <xsl:text>xmb_Sax.skipCurrentNode();</xsl:text><xsl:value-of select="$endl"/>
            </xsl:if>
        </xsl:if>
        <xsl:if test="$model2xml = 1">
            <xsl:call-template name="indent0"/> <xsl:text>if (true) {</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent1"/>     <xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/>
                                                        <xsl:text> = createNode(</xsl:text>
                                                            <xsl:text>xml_FocusNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>, </xsl:text>
                                                            <xsl:text>"</xsl:text><xsl:value-of select="@name"/><xsl:text>"</xsl:text>
                                                        <xsl:text>);</xsl:text><xsl:value-of select="$endl"/>

                                                    <xsl:for-each select="*"><xsl:apply-templates select="."/></xsl:for-each>

            <xsl:call-template name="indent0"/> <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
        </xsl:if>
    </xsl:template>

    <xsl:template match="xsd:attribute">
        <xsl:call-template name="traceTemplate"/>

        <xsl:if test="$xml2model = 1">
            <xsl:call-template name="indent0"/> <xsl:text>} else if (</xsl:text>
                                                    <xsl:if test="$dom = 1"><xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>.getNodeName()</xsl:text></xsl:if>
                                                    <xsl:if test="$sax = 1"><xsl:text>j_AttrQName</xsl:text><xsl:call-template name="makeVarId"/></xsl:if>
                                                    <xsl:text>.equals(</xsl:text>
                                                        <xsl:text>"</xsl:text><xsl:value-of select="@name"/><xsl:text>"</xsl:text>
                                                    <xsl:text>)</xsl:text>
                                                <xsl:text>) {</xsl:text><xsl:value-of select="$endl"/>

                                                    <xsl:for-each select="*"><xsl:apply-templates select="."/></xsl:for-each>
        </xsl:if>
        <xsl:if test="$model2xml = 1">
            <xsl:call-template name="indent0"/> <xsl:text>if (true) {</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent1"/>     <xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/>
                                                        <xsl:text> = createAttribute(</xsl:text>
                                                            <xsl:text>xml_FocusNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>, </xsl:text>
                                                            <xsl:text>"</xsl:text><xsl:value-of select="@name"/><xsl:text>"</xsl:text>
                                                        <xsl:text>);</xsl:text><xsl:value-of select="$endl"/>

                                                    <xsl:for-each select="*"><xsl:apply-templates select="."/></xsl:for-each>

            <xsl:call-template name="indent0"/> <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
        </xsl:if>
    </xsl:template>

    <xsl:template match="xsd:enumeration">
        <xsl:call-template name="traceTemplate"/>

        <xsl:if test="$xml2model = 1">
            <xsl:call-template name="indent0"/> <xsl:text>if (</xsl:text>
                                                    <xsl:if test="$dom = 1"><xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>.getTextContent()</xsl:text></xsl:if>
                                                    <xsl:if test="$sax = 1"><xsl:text>xmb_Sax.getTextContent()</xsl:text></xsl:if>
                                                    <xsl:text>.equals("</xsl:text><xsl:value-of select="@value"/><xsl:text>")</xsl:text>
                                                <xsl:text>) {</xsl:text><xsl:value-of select="$endl"/>

                <xsl:for-each select="*"><xsl:apply-templates select="."/></xsl:for-each>

            <xsl:call-template name="indent0"/> <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
        </xsl:if>
        <xsl:if test="$model2xml = 1">
            <xsl:call-template name="indent0"/> <xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/>
                                                    <xsl:text>.setTextContent("</xsl:text><xsl:value-of select="@value"/><xsl:text>");</xsl:text><xsl:value-of select="$endl"/>
        </xsl:if>
    </xsl:template>

    <xsl:template match="xsd:sequence | xsd:all | xsd:choice | xsd:restriction">
        <xsl:call-template name="traceTemplate"/>

        <xsl:for-each select="*"><xsl:apply-templates select="."/></xsl:for-each>
    </xsl:template>

<!-- XMB templates -->
    <xsl:template match="xmb:xml2model">
        <xsl:call-template name="traceTemplate"/>

        <xsl:if test="$xml2model = 1">
            <xsl:for-each select="*"><xsl:apply-templates select="."/></xsl:for-each>
        </xsl:if>
    </xsl:template>

    <xsl:template match="xmb:model2xml">
        <xsl:call-template name="traceTemplate"/>

        <xsl:if test="$model2xml = 1">
            <xsl:for-each select="*"><xsl:apply-templates select="."/></xsl:for-each>
        </xsl:if>
    </xsl:template>

    <xsl:template match="xmb:call">
        <xsl:call-template name="traceTemplate"/>

        <xsl:call-template name="indent1"/> <xsl:call-template name="makeFunctionName"/>
                                            <xsl:text>(</xsl:text>
                                                <xsl:if test="$dom = 1"><xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/></xsl:if>
                                                <xsl:if test="$sax = 1"><xsl:text>xmb_Sax</xsl:text></xsl:if>
                                                <xsl:for-each select="xmb:param">
                                                    <xsl:text>, </xsl:text><xsl:value-of select="."/>
                                                </xsl:for-each>
                                            <xsl:text>);</xsl:text>
                                            <xsl:value-of select="$endl"/>
    </xsl:template>

    <xsl:template match="xmb:param">
        <xsl:call-template name="traceTemplate"/>
    </xsl:template>

    <xsl:template match="xmb:code">
        <xsl:call-template name="traceTemplate"/>

        <xsl:call-template name="indent1"/><xsl:apply-templates/><xsl:value-of select="$endl"/>
    </xsl:template>

    <xsl:template match="xmb:xml2model/xmb:code/xmb:value">
        <xsl:call-template name="traceTemplate"/>

        <xsl:if test="$dom = 1"><xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>.getTextContent()</xsl:text></xsl:if>
        <xsl:if test="$sax = 1"><xsl:text>xmb_Sax.getTextContent()</xsl:text></xsl:if>
    </xsl:template>
    <xsl:template match="xmb:model2xml/xmb:value">
        <xsl:call-template name="traceTemplate"/>

        <xsl:call-template name="indent1"/> <xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>.setTextContent(</xsl:text>
                                                <xsl:text>String.valueOf(</xsl:text>
                                                    <xsl:apply-templates/>
                                                <xsl:text>)</xsl:text>
                                            <xsl:text>);</xsl:text><xsl:value-of select="$endl"/>
    </xsl:template>

    <xsl:template match="xmb:xpath">
        <xsl:call-template name="traceTemplate"/>

        <xsl:text>xpath(</xsl:text>
            <xsl:if test="$dom = 1"><xsl:text>xml_CurrentNode</xsl:text><xsl:call-template name="makeVarId"/><xsl:text>, </xsl:text></xsl:if>
            <xsl:if test="$sax = 1"><xsl:text>xmb_Sax, </xsl:text></xsl:if>
            <xsl:text>"</xsl:text><xsl:value-of select="@select"/><xsl:text>"</xsl:text>
        <xsl:text>)</xsl:text>
    </xsl:template>

<!-- Default -->
    <xsl:template match="*">
        <xsl:text>/* Unknown node </xsl:text><xsl:value-of select="local-name()"/><xsl:text> */</xsl:text>
        <xsl:for-each select="*"><xsl:apply-templates select="."/></xsl:for-each>
    </xsl:template>

<!-- XML-tools functions -->
    <xsl:template name="xmlTools">
        <xsl:if test="$xml2model = 1">
            <xsl:call-template name="indent1"/> <xsl:text>protected static class Display {</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>public static void println(String J_Text) {</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent3"/>         <xsl:text>System.out.println(J_Text);</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent1"/> <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent1"/> <xsl:text>protected static class Trace {</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>public static void println(String J_Text) {</xsl:text><xsl:value-of select="$endl"/>
            <xsl:if test="$debug = 1"><xsl:call-template name="indent3"/>    <xsl:text>System.err.println(J_Text);</xsl:text><xsl:value-of select="$endl"/></xsl:if>
            <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>public static void printStackTrace(Exception e) {</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent3"/>         <xsl:text>e.printStackTrace();</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent1"/> <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>

            <xsl:if test="$sax = 1">
                <xsl:call-template name="indent1"/> <xsl:text>protected class SaxReader </xsl:text>
                                                        <xsl:text>extends org.xml.sax.helpers.DefaultHandler </xsl:text>
                                                        <xsl:text>implements java.lang.Runnable </xsl:text>
                                                        <xsl:text>{</xsl:text><xsl:value-of select="$endl"/>

                                                        <!-- Threading members -->
                <xsl:call-template name="indent2"/>     <xsl:text>private final Object m_jUser;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>private final java.lang.Thread m_jThread;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>private java.util.concurrent.SynchronousQueue&lt;Object&gt; m_jSyncQueue;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>private java.lang.Exception m_jThreadException;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>private String m_jThreadExceptionLabel;</xsl:text><xsl:value-of select="$endl"/>
                                                        <!-- Parsing members -->
                <xsl:call-template name="indent2"/>     <xsl:text>private final java.io.FileInputStream m_jXmlFile;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>private org.xml.sax.Locator m_xmlLocator;</xsl:text><xsl:value-of select="$endl"/>
                                                        <!-- State members -->
                <xsl:call-template name="indent2"/>     <xsl:text>private org.xml.sax.Attributes m_xmlAttributes;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>private int m_iAttributeIndex;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>private String m_jNextChildQName;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>private boolean m_bWaitingForNodeTextContent;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>private String m_jNodeTextContent;</xsl:text><xsl:value-of select="$endl"/>

                                                        <!-- Constructor -->
                <xsl:call-template name="indent2"/>     <xsl:text>public SaxReader(Object J_User, java.io.FileInputStream J_XmlFile) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_jUser = J_User;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_jThread = new java.lang.Thread(this);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_jSyncQueue = new java.util.concurrent.SynchronousQueue&lt;Object&gt;();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_jXmlFile = J_XmlFile;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_xmlLocator = null;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>setThreadStateMembers(null, 0, null, null);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>setUserStateMembers(null, 0, false);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>private void setThreadStateMembers(</xsl:text>
                                                            <xsl:text>org.xml.sax.Attributes XML_Attributes, int I_AttributeIndex, </xsl:text>
                                                            <xsl:text>String J_NextChildQName, </xsl:text>
                                                            <xsl:text>String J_NodeTextContent</xsl:text>
                                                        <xsl:text>) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_xmlAttributes = XML_Attributes;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_iAttributeIndex = I_AttributeIndex;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_jNextChildQName = J_NextChildQName;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_jNodeTextContent = J_NodeTextContent;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_jThreadException = null;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_jThreadExceptionLabel = null;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>private void setUserStateMembers(</xsl:text>
                                                            <xsl:text>org.xml.sax.Attributes XML_Attributes, int I_AttributeIndex, </xsl:text>
                                                            <xsl:text>boolean B_WaitingForNodeTextContent</xsl:text>
                                                        <xsl:text>) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_iAttributeIndex = I_AttributeIndex;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_bWaitingForNodeTextContent = B_WaitingForNodeTextContent;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>

                                                        <!-- Threaded SAX parsing -->
                <xsl:call-template name="indent2"/>     <xsl:text>public void startParsing() {</xsl:text><xsl:value-of select="$endl"/>
                                                            <!-- Launch the thread -->
                <xsl:call-template name="indent3"/>         <xsl:text>m_jThread.start();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>getInitialExecToken();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>public void stopParsing() {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>Trace.println("stopParsing()");</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>stopThread();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>public void run() {</xsl:text><xsl:value-of select="$endl"/>
                                                            <!-- The main thread initially owns the execution token -->
                <xsl:call-template name="indent3"/>         <xsl:text>waitExecToken();</xsl:text><xsl:value-of select="$endl"/>
                                                            <!-- Launch SAX parsing -->
                <xsl:call-template name="indent3"/>         <xsl:text>try {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>javax.xml.parsers.SAXParserFactory xml_SAXFactory = </xsl:text>
                                                                    <xsl:text>javax.xml.parsers.SAXParserFactory.newInstance();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>javax.xml.parsers.SAXParser xml_Parser = </xsl:text>
                                                                    <xsl:text>xml_SAXFactory.newSAXParser();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>xml_Parser.parse(m_jXmlFile, this);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>} catch (org.xml.sax.SAXParseException j_SaxException) {</xsl:text><xsl:value-of select="$endl"/>
                                                                <!-- SAX errors should normally have already been displayed -->
                <xsl:call-template name="indent3"/>         <xsl:text>} catch (java.lang.UnsupportedOperationException j_UnsupportedOperationException) {</xsl:text><xsl:value-of select="$endl"/>
                                                                <!-- No need to display java.lang.UnsupportedOperationException -->
                <xsl:call-template name="indent3"/>         <xsl:text>} catch (Exception j_Exception) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>reportException(null, j_Exception);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>

                                                        <!-- User interface -->
                <xsl:call-template name="indent2"/>     <xsl:text>public org.xml.sax.Attributes getCurrentAttributes() {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>return m_xmlAttributes;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>public String getNextAttributeQName() {</xsl:text><xsl:value-of select="$endl"/>
                                                            <!-- No need to wait as far as we are already positionned on a node -->
                <xsl:call-template name="indent3"/>         <xsl:text>if ((m_xmlAttributes != null) </xsl:text>
                                                                <xsl:text>&amp;&amp; (m_iAttributeIndex + 1 &lt; m_xmlAttributes.getLength())) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>setUserStateMembers(m_xmlAttributes, m_iAttributeIndex + 1, false);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>Trace.println("SAX: getNextAttributeQName() -> " + m_xmlAttributes.getQName(m_iAttributeIndex));</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>return m_xmlAttributes.getQName(m_iAttributeIndex);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>} else {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>setUserStateMembers(null, 0, false);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>Trace.println("SAX: getNextAttributeQName() -> null");</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>return null;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>public String getNextChildQName() {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>setUserStateMembers(null, 0, false);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>releaseExecToken();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>waitExecToken();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>Trace.println("SAX: getNextChildQName() -> " + m_jNextChildQName);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>return m_jNextChildQName;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>void skipCurrentNode() {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>Trace.println("SAX: skipCurrentNode() ...");</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>for (int i_Stacked = 1; i_Stacked &gt; 0; ) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>if (getNextChildQName() != null) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent5"/>                 <xsl:text>i_Stacked ++;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>} else {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent5"/>                 <xsl:text>i_Stacked --;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>Trace.println("SAX: skipCurrentNode() ->");</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>public String getTextContent() {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>if ((m_xmlAttributes != null) </xsl:text>
                                                                <xsl:text>&amp;&amp; (m_iAttributeIndex &gt;= 0) &amp;&amp; (m_iAttributeIndex &lt; m_xmlAttributes.getLength())</xsl:text>
                                                                <xsl:text>) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>Trace.println("SAX: getTextContent(attribute) -> " + m_xmlAttributes.getValue(m_iAttributeIndex));</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>return m_xmlAttributes.getValue(m_iAttributeIndex);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>} else {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>setUserStateMembers(null, 0, true);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>releaseExecToken();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>waitExecToken();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>if (m_jNodeTextContent != null) {</xsl:text>
                <xsl:call-template name="indent5"/>                 <xsl:text>Trace.println("SAX: getTextContent(node) -> " + m_jNodeTextContent);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent5"/>                 <xsl:text>return m_jNodeTextContent;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>} else {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent5"/>                 <xsl:text>Trace.println("SAX: getTextContent(node) -> new String()");</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent5"/>                 <xsl:text>return new String();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>

                                                        <!-- SAX interface -->
                <xsl:call-template name="indent2"/>     <xsl:text>public void setDocumentLocator(org.xml.sax.Locator locator) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_xmlLocator = locator;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>public void startElement(</xsl:text>
                                                            <xsl:text>String uri, String localName, String qName, org.xml.sax.Attributes attributes</xsl:text>
                                                            <xsl:text>) throws org.xml.sax.SAXException {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>Trace.println("SAX: startElement(" </xsl:text>
                                                                <xsl:text>+ "uri = " + uri + ", " </xsl:text>
                                                                <xsl:text>+ "localName = " + localName + ", " </xsl:text>
                                                                <xsl:text>+ "qName = " + qName + ", " </xsl:text>
                                                                <xsl:text>+ "attributes = " + attributes </xsl:text>
                                                            <xsl:text>+ ")");</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>setThreadStateMembers(attributes, -1, qName, null);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>releaseExecToken();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>waitExecToken();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>public void endElement(String uri, String localName, String qName) </xsl:text>
                                                            <xsl:text>throws org.xml.sax.SAXException {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>Trace.println("SAX: endElement(" </xsl:text>
                                                                <xsl:text>+ "uri = " + uri + ", " </xsl:text>
                                                                <xsl:text>+ "localName = " + localName + ", " </xsl:text>
                                                                <xsl:text>+ "qName = " + qName </xsl:text>
                                                            <xsl:text>+ ")");</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>setThreadStateMembers(null, 0, null, null);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>releaseExecToken();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>waitExecToken();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>public void characters(char[] ch, int start, int length) </xsl:text>
                                                            <xsl:text>throws org.xml.sax.SAXException {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>if (m_bWaitingForNodeTextContent) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>Trace.println("SAX: characters(" </xsl:text>
                                                                    <xsl:text>+ "text = " + new String(ch, start, length) </xsl:text>
                                                                <xsl:text>+ ")");</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>setThreadStateMembers(null, 0, null, new String(ch, start, length));</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>releaseExecToken();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>waitExecToken();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                                                        <!-- Receive notification of a parser warning -->
                <xsl:call-template name="indent2"/>     <xsl:text>public void warning(org.xml.sax.SAXParseException e) </xsl:text>
                                                            <xsl:text>throws org.xml.sax.SAXException {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>reportException("Warning", e);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>super.warning(e);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                                                        <!-- Receive notification of a recoverable parser error -->
                <xsl:call-template name="indent2"/>     <xsl:text>public void error(org.xml.sax.SAXParseException e) </xsl:text>
                                                            <xsl:text>throws org.xml.sax.SAXException {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>reportException("Error", e);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>super.error(e);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>private void reportException(String J_Label, java.lang.Exception J_Exception) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_jThreadException = J_Exception;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_jThreadExceptionLabel = J_Label;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>releaseExecToken();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>waitExecToken();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                                                        <!-- Report a fatal XML parsing error -->
                <xsl:call-template name="indent2"/>     <xsl:text>public void fatalError(org.xml.sax.SAXParseException e) </xsl:text>
                                                            <xsl:text>throws org.xml.sax.SAXException {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>reportException("Fatal error", e);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>super.fatalError(e);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>

                                                        <!-- Thread synchronization -->
                <xsl:call-template name="indent2"/>     <xsl:text>private void getInitialExecToken() {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>private void waitExecToken() {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>try {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>do {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent5"/>                 <xsl:text>if (m_jThreadException != null) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent6"/>                     <xsl:text>if ((m_jThreadException instanceof org.xml.sax.SAXParseException) </xsl:text>
                                                                            <xsl:text>&amp;&amp; (m_jThreadExceptionLabel != null)) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent7"/>                         <xsl:text>org.xml.sax.SAXParseException j_ThreadException = (org.xml.sax.SAXParseException) m_jThreadException;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent7"/>                         <xsl:text>Display.println(</xsl:text>
                                                                                <xsl:text>new java.io.File(j_ThreadException.getSystemId()).getName() + ":" </xsl:text>
                                                                                <xsl:text>+ new Integer(j_ThreadException.getLineNumber()).toString() + ":" </xsl:text>
                                                                                <xsl:text>+ new Integer(j_ThreadException.getColumnNumber()).toString() + ": " </xsl:text>
                                                                                <xsl:text>+ m_jThreadExceptionLabel + "! " </xsl:text>
                                                                                <xsl:text>+ j_ThreadException.getMessage()</xsl:text>
                                                                            <xsl:text>);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent6"/>                     <xsl:text>} else {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent7"/>                         <xsl:text>Trace.printStackTrace(m_jThreadException);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent6"/>                     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent6"/>                     <xsl:text>m_jThreadException = null;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent6"/>                     <xsl:text>m_jThreadExceptionLabel = null;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent5"/>                 <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent5"/>                 <xsl:text>m_jSyncQueue.take();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent5"/>                 <xsl:text>if (m_jSyncQueue == null) {</xsl:text><xsl:value-of select="$endl"/>
                                                                        <!-- Stop thread execution right now -->
                <xsl:call-template name="indent6"/>                     <xsl:text>throw new java.lang.UnsupportedOperationException();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent5"/>                 <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>} while ((m_jThreadException != null) &amp;&amp; (m_jThreadExceptionLabel != null));</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>} catch (java.lang.InterruptedException j_InterruptedException) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>j_InterruptedException.printStackTrace();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>} catch (java.lang.NullPointerException j_NullPointerException) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>             <xsl:text>throw new java.lang.UnsupportedOperationException();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>private void releaseExecToken() {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>try {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>m_jSyncQueue.put(new Object());</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>} catch (java.lang.InterruptedException j_InterruptedException) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>j_InterruptedException.printStackTrace();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>} catch (java.lang.NullPointerException j_NullPointerException) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>throw new java.lang.UnsupportedOperationException();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>private void stopThread() {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>java.util.concurrent.SynchronousQueue&lt;Object&gt; j_SyncQueue = m_jSyncQueue;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>m_jSyncQueue = null;</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>try {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>j_SyncQueue.put(new Object());</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>} catch (java.lang.InterruptedException j_InterruptedException) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent4"/>             <xsl:text>j_InterruptedException.printStackTrace();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>

                <xsl:call-template name="indent1"/> <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                <xsl:value-of select="$endl"/>
            </xsl:if>

            <xsl:call-template name="indent1"/> <xsl:text>protected String xpath(</xsl:text>
                                                    <xsl:if test="$dom = 1"><xsl:text>org.w3c.dom.Node XML_Node, </xsl:text></xsl:if>
                                                    <xsl:if test="$sax = 1"><xsl:text>SaxReader XMB_Sax, </xsl:text></xsl:if>
                                                    <xsl:text>String J_Expression) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>try {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:if test="$dom = 1">
                    <xsl:call-template name="indent3"/>     <!-- Factory creation -->
                                                            <xsl:text>javax.xml.xpath.XPathFactory xml_Factory = javax.xml.xpath.XPathFactory.newInstance();</xsl:text><xsl:value-of select="$endl"/>
                    <xsl:call-template name="indent3"/>     <!-- XPath object creation -->
                                                            <xsl:text>javax.xml.xpath.XPath xml_XPath = xml_Factory.newXPath();</xsl:text><xsl:value-of select="$endl"/>
                    <xsl:call-template name="indent3"/>     <!-- XPath expression compilation and evaluation -->
                                                            <xsl:text>String j_Res = xml_XPath.evaluate(</xsl:text>
                                                                <xsl:text>J_Expression, </xsl:text>
                                                                <xsl:text>XML_Node</xsl:text>
                                                            <xsl:text>);</xsl:text><xsl:value-of select="$endl"/>
                    <xsl:call-template name="indent3"/>     <xsl:text>return j_Res;</xsl:text><xsl:value-of select="$endl"/>
                </xsl:if>
                <xsl:if test="$sax = 1">
                    <xsl:call-template name="indent3"/>     <!-- Only attribute xpath expression are managed -->
                                                            <xsl:text>if (J_Expression.charAt(0) == '@') {</xsl:text><xsl:value-of select="$endl"/>
                    <xsl:call-template name="indent4"/>         <xsl:text>String j_AttributeName = J_Expression.substring(1);</xsl:text><xsl:value-of select="$endl"/>
                    <xsl:call-template name="indent4"/>         <xsl:text>Trace.println("SAX: xpath(" + J_Expression + ") -> " + XMB_Sax.getCurrentAttributes().getValue(j_AttributeName));</xsl:text><xsl:value-of select="$endl"/>
                    <xsl:call-template name="indent4"/>         <xsl:text>return XMB_Sax.getCurrentAttributes().getValue(j_AttributeName);</xsl:text><xsl:value-of select="$endl"/>
                    <xsl:call-template name="indent3"/>     <xsl:text>} else {</xsl:text><xsl:value-of select="$endl"/>
                    <xsl:call-template name="indent4"/>         <xsl:text>Trace.println("SAX: xpath(" + J_Expression + ") -> new String()");</xsl:text><xsl:value-of select="$endl"/>
                    <xsl:call-template name="indent4"/>         <xsl:text>return new String();</xsl:text><xsl:value-of select="$endl"/>
                    <xsl:call-template name="indent3"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
                </xsl:if>
                <xsl:call-template name="indent2"/>     <xsl:text>} catch (Exception j_Exception) {</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>Trace.printStackTrace(j_Exception);</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent3"/>         <xsl:text>return new String();</xsl:text><xsl:value-of select="$endl"/>
                <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent1"/> <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
            <xsl:value-of select="$endl"/>
        </xsl:if>
        <xsl:if test="$model2xml = 1">
            <xsl:call-template name="indent1"/> <xsl:text>protected org.w3c.dom.Element createNode(org.w3c.dom.Node XML_Node, String J_NodeName) {</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>try {</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent3"/>         <xsl:text>org.w3c.dom.Document xml_Doc = XML_Node.getOwnerDocument();</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent3"/>         <xsl:text>org.w3c.dom.Element xml_CurrentNode = xml_Doc.createElement(J_NodeName);</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent3"/>         <xsl:text>XML_Node.appendChild(xml_CurrentNode);</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent3"/>         <xsl:text>return xml_CurrentNode;</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>} catch (Exception j_Exception) {</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent3"/>         <xsl:text>Trace.printStackTrace(j_Exception);</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent3"/>         <xsl:text>return null;</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent1"/> <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
            <xsl:value-of select="$endl"/>

            <xsl:call-template name="indent1"/> <xsl:text>protected org.w3c.dom.Attr createAttribute(org.w3c.dom.Node XML_Node, String J_AttributeName) {</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>try {</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent3"/>         <xsl:text>org.w3c.dom.Document xml_Doc = XML_Node.getOwnerDocument();</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent3"/>         <xsl:text>org.w3c.dom.Attr xml_Attribute = xml_Doc.createAttribute(J_AttributeName);</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent3"/>         <xsl:text>XML_Node.getAttributes().setNamedItem(xml_Attribute);</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent3"/>         <xsl:text>return xml_Attribute;</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>} catch (Exception j_Exception) {</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent3"/>         <xsl:text>Trace.printStackTrace(j_Exception);</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent3"/>         <xsl:text>return null;</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent2"/>     <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
            <xsl:call-template name="indent1"/> <xsl:text>}</xsl:text><xsl:value-of select="$endl"/>
            <xsl:value-of select="$endl"/>
        </xsl:if>
    </xsl:template>

</xsl:stylesheet>

